@@ -24,21 +24,21 @@
 #define INCLUDED_PMT_H
 
 #include <pmt/api.h>
-#include <boost/intrusive_ptr.hpp>
-#include <boost/shared_ptr.hpp>
-#include <boost/any.hpp>
-#include <complex>
-#include <string>
 #include <stdint.h>
+#include <boost/any.hpp>
+#include <boost/noncopyable.hpp>
+#include <boost/shared_ptr.hpp>
+#include <complex>
 #include <iosfwd>
 #include <stdexcept>
+#include <string>
 #include <vector>
 
 namespace gr {
-  namespace messages {
-    class msg_accepter;
-  }
+namespace messages {
+class msg_accepter;
 }
+} // namespace gr
 
 /*!
  * This file defines a polymorphic type and the operations on it.
@@ -53,39 +53,70 @@ namespace pmt {
 /*!
  * \brief base class of all pmt types
  */
-class pmt_base;
+class pmt_base : boost::noncopyable
+{
+
+public:
+    pmt_base(){};
+    virtual ~pmt_base();
+
+    virtual bool is_bool() const { return false; }
+    virtual bool is_symbol() const { return false; }
+    virtual bool is_number() const { return false; }
+    virtual bool is_integer() const { return false; }
+    virtual bool is_uint64() const { return false; }
+    virtual bool is_real() const { return false; }
+    virtual bool is_complex() const { return false; }
+    virtual bool is_null() const { return false; }
+    virtual bool is_pair() const { return false; }
+    virtual bool is_tuple() const { return false; }
+    virtual bool is_vector() const { return false; }
+    virtual bool is_dict() const { return false; }
+    virtual bool is_any() const { return false; }
+
+    virtual bool is_uniform_vector() const { return false; }
+    virtual bool is_u8vector() const { return false; }
+    virtual bool is_s8vector() const { return false; }
+    virtual bool is_u16vector() const { return false; }
+    virtual bool is_s16vector() const { return false; }
+    virtual bool is_u32vector() const { return false; }
+    virtual bool is_s32vector() const { return false; }
+    virtual bool is_u64vector() const { return false; }
+    virtual bool is_s64vector() const { return false; }
+    virtual bool is_f32vector() const { return false; }
+    virtual bool is_f64vector() const { return false; }
+    virtual bool is_c32vector() const { return false; }
+    virtual bool is_c64vector() const { return false; }
+};
 
 /*!
  * \brief typedef for shared pointer (transparent reference counting).
  * See http://www.boost.org/libs/smart_ptr/smart_ptr.htm
  */
-typedef boost::intrusive_ptr<pmt_base> pmt_t;
-
-extern PMT_API void intrusive_ptr_add_ref(pmt_base*);
-extern PMT_API void intrusive_ptr_release(pmt_base*);
+typedef boost::shared_ptr<pmt_base> pmt_t;
 
 class PMT_API exception : public std::logic_error
 {
 public:
-  exception(const std::string &msg, pmt_t obj);
+    exception(const std::string& msg, pmt_t obj);
 };
 
 class PMT_API wrong_type : public exception
 {
 public:
-  wrong_type(const std::string &msg, pmt_t obj);
+    wrong_type(const std::string& msg, pmt_t obj);
 };
 
 class PMT_API out_of_range : public exception
 {
 public:
-  out_of_range(const std::string &msg, pmt_t obj);
+    out_of_range(const std::string& msg, pmt_t obj);
 };
 
 class PMT_API notimplemented : public exception
 {
 public:
-  notimplemented(const std::string &msg, pmt_t obj);
+    notimplemented(const std::string& msg, pmt_t obj);
 };
 
 
@@ -106,7 +137,6 @@ PMT_API pmt_t get_PMT_EOF();
 #define PMT_EOF get_PMT_EOF()
 
 
-
 /*
  * ------------------------------------------------------------------------
  * Booleans.  Two constants, #t and #f.
@@ -142,10 +172,10 @@ PMT_API bool to_bool(pmt_t val);
 PMT_API bool is_symbol(const pmt_t& obj);
 
 //! Return the symbol whose name is \p s.
-PMT_API pmt_t string_to_symbol(const std::string &s);
+PMT_API pmt_t string_to_symbol(const std::string& s);
 
 //! Alias for pmt_string_to_symbol
-PMT_API pmt_t intern(const std::string &s);
+PMT_API pmt_t intern(const std::string& s);
 
 
 /*!
@@ -256,13 +286,13 @@ PMT_API pmt_t make_rectangular(double re, double im);
 PMT_API pmt_t from_complex(double re, double im);
 
 //! Return a complex number constructed of the given a complex number.
-PMT_API pmt_t from_complex(const std::complex<double> &z);
+PMT_API pmt_t from_complex(const std::complex<double>& z);
 
 //! Return a complex number constructed of the given real and imaginary parts.
 PMT_API pmt_t pmt_from_complex(double re, double im);
 
 //! Return a complex number constructed of the given a complex number.
-PMT_API pmt_t pmt_from_complex(const std::complex<double> &z);
+PMT_API pmt_t pmt_from_complex(const std::complex<double>& z);
 
 /*!
  * If \p z is complex, real or integer, return the closest complex<double>.
@@ -318,27 +348,66 @@ PMT_API pmt_t cadddr(pmt_t pair);
 PMT_API bool is_tuple(pmt_t x);
 
 PMT_API pmt_t make_tuple();
-PMT_API pmt_t make_tuple(const pmt_t &e0);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3, const pmt_t &e4);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3, const pmt_t &e4, const pmt_t &e5);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3, const pmt_t &e4, const pmt_t &e5, const pmt_t &e6);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3, const pmt_t &e4, const pmt_t &e5, const pmt_t &e6, const pmt_t &e7);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3, const pmt_t &e4, const pmt_t &e5, const pmt_t &e6, const pmt_t &e7, const pmt_t &e8);
-PMT_API pmt_t make_tuple(const pmt_t &e0, const pmt_t &e1, const pmt_t &e2, const pmt_t &e3, const pmt_t &e4, const pmt_t &e5, const pmt_t &e6, const pmt_t &e7, const pmt_t &e8, const pmt_t &e9);
+PMT_API pmt_t make_tuple(const pmt_t& e0);
+PMT_API pmt_t make_tuple(const pmt_t& e0, const pmt_t& e1);
+PMT_API pmt_t make_tuple(const pmt_t& e0, const pmt_t& e1, const pmt_t& e2);
+PMT_API pmt_t make_tuple(const pmt_t& e0,
+                         const pmt_t& e1,
+                         const pmt_t& e2,
+                         const pmt_t& e3);
+PMT_API pmt_t make_tuple(
+    const pmt_t& e0, const pmt_t& e1, const pmt_t& e2, const pmt_t& e3, const pmt_t& e4);
+PMT_API pmt_t make_tuple(const pmt_t& e0,
+                         const pmt_t& e1,
+                         const pmt_t& e2,
+                         const pmt_t& e3,
+                         const pmt_t& e4,
+                         const pmt_t& e5);
+PMT_API pmt_t make_tuple(const pmt_t& e0,
+                         const pmt_t& e1,
+                         const pmt_t& e2,
+                         const pmt_t& e3,
+                         const pmt_t& e4,
+                         const pmt_t& e5,
+                         const pmt_t& e6);
+PMT_API pmt_t make_tuple(const pmt_t& e0,
+                         const pmt_t& e1,
+                         const pmt_t& e2,
+                         const pmt_t& e3,
+                         const pmt_t& e4,
+                         const pmt_t& e5,
+                         const pmt_t& e6,
+                         const pmt_t& e7);
+PMT_API pmt_t make_tuple(const pmt_t& e0,
+                         const pmt_t& e1,
+                         const pmt_t& e2,
+                         const pmt_t& e3,
+                         const pmt_t& e4,
+                         const pmt_t& e5,
+                         const pmt_t& e6,
+                         const pmt_t& e7,
+                         const pmt_t& e8);
+PMT_API pmt_t make_tuple(const pmt_t& e0,
+                         const pmt_t& e1,
+                         const pmt_t& e2,
+                         const pmt_t& e3,
+                         const pmt_t& e4,
+                         const pmt_t& e5,
+                         const pmt_t& e6,
+                         const pmt_t& e7,
+                         const pmt_t& e8,
+                         const pmt_t& e9);
 
 /*!
  * If \p x is a vector or proper list, return a tuple containing the elements of x
  */
-PMT_API pmt_t to_tuple(const pmt_t &x);
+PMT_API pmt_t to_tuple(const pmt_t& x);
 
 /*!
  * Return the contents of position \p k of \p tuple.
  * \p k must be a valid index of \p tuple.
  */
-PMT_API pmt_t tuple_ref(const pmt_t &tuple, size_t k);
+PMT_API pmt_t tuple_ref(const pmt_t& tuple, size_t k);
 
 /*
  * ------------------------------------------------------------------------
@@ -385,10 +454,10 @@ PMT_API bool is_blob(pmt_t x);
  *
  * The data is copied into the blob.
  */
-PMT_API pmt_t make_blob(const void *buf, size_t len);
+PMT_API pmt_t make_blob(const void* buf, size_t len);
 
 //! Return a pointer to the blob's data
-PMT_API const void *blob_data(pmt_t blob);
+PMT_API const void* blob_data(pmt_t blob);
 
 //! Return the blob's length in bytes
 PMT_API size_t blob_length(pmt_t blob);
@@ -452,45 +521,45 @@ PMT_API pmt_t make_f64vector(size_t k, double fill);
 PMT_API pmt_t make_c32vector(size_t k, std::complex<float> fill);
 PMT_API pmt_t make_c64vector(size_t k, std::complex<double> fill);
 
-PMT_API pmt_t init_u8vector(size_t k, const uint8_t *data);
-PMT_API pmt_t init_u8vector(size_t k, const std::vector<uint8_t> &data);
-PMT_API pmt_t init_s8vector(size_t k, const int8_t *data);
-PMT_API pmt_t init_s8vector(size_t k, const std::vector<int8_t> &data);
-PMT_API pmt_t init_u16vector(size_t k, const uint16_t *data);
-PMT_API pmt_t init_u16vector(size_t k, const std::vector<uint16_t> &data);
-PMT_API pmt_t init_s16vector(size_t k, const int16_t *data);
-PMT_API pmt_t init_s16vector(size_t k, const std::vector<int16_t> &data);
-PMT_API pmt_t init_u32vector(size_t k, const uint32_t *data);
-PMT_API pmt_t init_u32vector(size_t k, const std::vector<uint32_t> &data);
-PMT_API pmt_t init_s32vector(size_t k, const int32_t *data);
-PMT_API pmt_t init_s32vector(size_t k, const std::vector<int32_t> &data);
-PMT_API pmt_t init_u64vector(size_t k, const uint64_t *data);
-PMT_API pmt_t init_u64vector(size_t k, const std::vector<uint64_t> &data);
-PMT_API pmt_t init_s64vector(size_t k, const int64_t *data);
-PMT_API pmt_t init_s64vector(size_t k, const std::vector<int64_t> &data);
-PMT_API pmt_t init_f32vector(size_t k, const float *data);
-PMT_API pmt_t init_f32vector(size_t k, const std::vector<float> &data);
-PMT_API pmt_t init_f64vector(size_t k, const double *data);
-PMT_API pmt_t init_f64vector(size_t k, const std::vector<double> &data);
-PMT_API pmt_t init_c32vector(size_t k, const std::complex<float> *data);
-PMT_API pmt_t init_c32vector(size_t k, const std::vector<std::complex<float> > &data);
-PMT_API pmt_t init_c64vector(size_t k, const std::complex<double> *data);
-PMT_API pmt_t init_c64vector(size_t k, const std::vector<std::complex<double> > &data);
+PMT_API pmt_t init_u8vector(size_t k, const uint8_t* data);
+PMT_API pmt_t init_u8vector(size_t k, const std::vector<uint8_t>& data);
+PMT_API pmt_t init_s8vector(size_t k, const int8_t* data);
+PMT_API pmt_t init_s8vector(size_t k, const std::vector<int8_t>& data);
+PMT_API pmt_t init_u16vector(size_t k, const uint16_t* data);
+PMT_API pmt_t init_u16vector(size_t k, const std::vector<uint16_t>& data);
+PMT_API pmt_t init_s16vector(size_t k, const int16_t* data);
+PMT_API pmt_t init_s16vector(size_t k, const std::vector<int16_t>& data);
+PMT_API pmt_t init_u32vector(size_t k, const uint32_t* data);
+PMT_API pmt_t init_u32vector(size_t k, const std::vector<uint32_t>& data);
+PMT_API pmt_t init_s32vector(size_t k, const int32_t* data);
+PMT_API pmt_t init_s32vector(size_t k, const std::vector<int32_t>& data);
+PMT_API pmt_t init_u64vector(size_t k, const uint64_t* data);
+PMT_API pmt_t init_u64vector(size_t k, const std::vector<uint64_t>& data);
+PMT_API pmt_t init_s64vector(size_t k, const int64_t* data);
+PMT_API pmt_t init_s64vector(size_t k, const std::vector<int64_t>& data);
+PMT_API pmt_t init_f32vector(size_t k, const float* data);
+PMT_API pmt_t init_f32vector(size_t k, const std::vector<float>& data);
+PMT_API pmt_t init_f64vector(size_t k, const double* data);
+PMT_API pmt_t init_f64vector(size_t k, const std::vector<double>& data);
+PMT_API pmt_t init_c32vector(size_t k, const std::complex<float>* data);
+PMT_API pmt_t init_c32vector(size_t k, const std::vector<std::complex<float>>& data);
+PMT_API pmt_t init_c64vector(size_t k, const std::complex<double>* data);
+PMT_API pmt_t init_c64vector(size_t k, const std::vector<std::complex<double>>& data);
 
-PMT_API uint8_t  u8vector_ref(pmt_t v, size_t k);
-PMT_API int8_t   s8vector_ref(pmt_t v, size_t k);
+PMT_API uint8_t u8vector_ref(pmt_t v, size_t k);
+PMT_API int8_t s8vector_ref(pmt_t v, size_t k);
 PMT_API uint16_t u16vector_ref(pmt_t v, size_t k);
-PMT_API int16_t  s16vector_ref(pmt_t v, size_t k);
+PMT_API int16_t s16vector_ref(pmt_t v, size_t k);
 PMT_API uint32_t u32vector_ref(pmt_t v, size_t k);
-PMT_API int32_t  s32vector_ref(pmt_t v, size_t k);
+PMT_API int32_t s32vector_ref(pmt_t v, size_t k);
 PMT_API uint64_t u64vector_ref(pmt_t v, size_t k);
-PMT_API int64_t  s64vector_ref(pmt_t v, size_t k);
-PMT_API float    f32vector_ref(pmt_t v, size_t k);
-PMT_API double   f64vector_ref(pmt_t v, size_t k);
-PMT_API std::complex<float>  c32vector_ref(pmt_t v, size_t k);
+PMT_API int64_t s64vector_ref(pmt_t v, size_t k);
+PMT_API float f32vector_ref(pmt_t v, size_t k);
+PMT_API double f64vector_ref(pmt_t v, size_t k);
+PMT_API std::complex<float> c32vector_ref(pmt_t v, size_t k);
 PMT_API std::complex<double> c64vector_ref(pmt_t v, size_t k);
 
-PMT_API void u8vector_set(pmt_t v, size_t k, uint8_t x);  //< v[k] = x
+PMT_API void u8vector_set(pmt_t v, size_t k, uint8_t x); //< v[k] = x
 PMT_API void s8vector_set(pmt_t v, size_t k, int8_t x);
 PMT_API void u16vector_set(pmt_t v, size_t k, uint16_t x);
 PMT_API void s16vector_set(pmt_t v, size_t k, int16_t x);
@@ -505,65 +574,75 @@ PMT_API void c64vector_set(pmt_t v, size_t k, std::complex<double> x);
 
 // Return const pointers to the elements
 
-PMT_API const void *uniform_vector_elements(pmt_t v, size_t &len);  //< works with any; len is in bytes
+PMT_API const void*
+uniform_vector_elements(pmt_t v, size_t& len); //< works with any; len is in bytes
 
-PMT_API const uint8_t  *u8vector_elements(pmt_t v, size_t &len);  //< len is in elements
-PMT_API const int8_t   *s8vector_elements(pmt_t v, size_t &len);  //< len is in elements
-PMT_API const uint16_t *u16vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const int16_t  *s16vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const uint32_t *u32vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const int32_t  *s32vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const uint64_t *u64vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const int64_t  *s64vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const float    *f32vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const double   *f64vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const std::complex<float>  *c32vector_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API const std::complex<double> *c64vector_elements(pmt_t v, size_t &len); //< len is in elements
+PMT_API const uint8_t* u8vector_elements(pmt_t v, size_t& len);   //< len is in elements
+PMT_API const int8_t* s8vector_elements(pmt_t v, size_t& len);    //< len is in elements
+PMT_API const uint16_t* u16vector_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API const int16_t* s16vector_elements(pmt_t v, size_t& len);  //< len is in elements
+PMT_API const uint32_t* u32vector_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API const int32_t* s32vector_elements(pmt_t v, size_t& len);  //< len is in elements
+PMT_API const uint64_t* u64vector_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API const int64_t* s64vector_elements(pmt_t v, size_t& len);  //< len is in elements
+PMT_API const float* f32vector_elements(pmt_t v, size_t& len);    //< len is in elements
+PMT_API const double* f64vector_elements(pmt_t v, size_t& len);   //< len is in elements
+PMT_API const std::complex<float>* c32vector_elements(pmt_t v,
+                                                      size_t& len); //< len is in elements
+PMT_API const std::complex<double>*
+c64vector_elements(pmt_t v, size_t& len); //< len is in elements
 
 // len is in elements
-PMT_API const std::vector<uint8_t>  u8vector_elements(pmt_t v);
-PMT_API const std::vector<int8_t>   s8vector_elements(pmt_t v);
+PMT_API const std::vector<uint8_t> u8vector_elements(pmt_t v);
+PMT_API const std::vector<int8_t> s8vector_elements(pmt_t v);
 PMT_API const std::vector<uint16_t> u16vector_elements(pmt_t v);
-PMT_API const std::vector<int16_t>  s16vector_elements(pmt_t v);
+PMT_API const std::vector<int16_t> s16vector_elements(pmt_t v);
 PMT_API const std::vector<uint32_t> u32vector_elements(pmt_t v);
-PMT_API const std::vector<int32_t>  s32vector_elements(pmt_t v);
+PMT_API const std::vector<int32_t> s32vector_elements(pmt_t v);
 PMT_API const std::vector<uint64_t> u64vector_elements(pmt_t v);
-PMT_API const std::vector<int64_t>  s64vector_elements(pmt_t v);
-PMT_API const std::vector<float>    f32vector_elements(pmt_t v);
-PMT_API const std::vector<double>   f64vector_elements(pmt_t v);
-PMT_API const std::vector<std::complex<float> > c32vector_elements(pmt_t v);
-PMT_API const std::vector<std::complex<double> > c64vector_elements(pmt_t v);
+PMT_API const std::vector<int64_t> s64vector_elements(pmt_t v);
+PMT_API const std::vector<float> f32vector_elements(pmt_t v);
+PMT_API const std::vector<double> f64vector_elements(pmt_t v);
+PMT_API const std::vector<std::complex<float>> c32vector_elements(pmt_t v);
+PMT_API const std::vector<std::complex<double>> c64vector_elements(pmt_t v);
 
 // len is in elements
-PMT_API const std::vector<uint8_t>  pmt_u8vector_elements(pmt_t v);
-PMT_API const std::vector<int8_t>   pmt_s8vector_elements(pmt_t v);
+PMT_API const std::vector<uint8_t> pmt_u8vector_elements(pmt_t v);
+PMT_API const std::vector<int8_t> pmt_s8vector_elements(pmt_t v);
 PMT_API const std::vector<uint16_t> pmt_u16vector_elements(pmt_t v);
-PMT_API const std::vector<int16_t>  pmt_s16vector_elements(pmt_t v);
+PMT_API const std::vector<int16_t> pmt_s16vector_elements(pmt_t v);
 PMT_API const std::vector<uint32_t> pmt_u32vector_elements(pmt_t v);
-PMT_API const std::vector<int32_t>  pmt_s32vector_elements(pmt_t v);
+PMT_API const std::vector<int32_t> pmt_s32vector_elements(pmt_t v);
 PMT_API const std::vector<uint64_t> pmt_u64vector_elements(pmt_t v);
-PMT_API const std::vector<int64_t>  pmt_s64vector_elements(pmt_t v);
-PMT_API const std::vector<float>    pmt_f32vector_elements(pmt_t v);
-PMT_API const std::vector<double>   pmt_f64vector_elements(pmt_t v);
-PMT_API const std::vector<std::complex<float> > pmt_c32vector_elements(pmt_t v);
-PMT_API const std::vector<std::complex<double> > pmt_c64vector_elements(pmt_t v);
+PMT_API const std::vector<int64_t> pmt_s64vector_elements(pmt_t v);
+PMT_API const std::vector<float> pmt_f32vector_elements(pmt_t v);
+PMT_API const std::vector<double> pmt_f64vector_elements(pmt_t v);
+PMT_API const std::vector<std::complex<float>> pmt_c32vector_elements(pmt_t v);
+PMT_API const std::vector<std::complex<double>> pmt_c64vector_elements(pmt_t v);
 
 // Return non-const pointers to the elements
 
-PMT_API void *uniform_vector_writable_elements(pmt_t v, size_t &len);  //< works with any; len is in bytes
+PMT_API void*
+uniform_vector_writable_elements(pmt_t v,
+                                 size_t& len); //< works with any; len is in bytes
 
-PMT_API uint8_t  *u8vector_writable_elements(pmt_t v, size_t &len);  //< len is in elements
-PMT_API int8_t   *s8vector_writable_elements(pmt_t v, size_t &len);  //< len is in elements
-PMT_API uint16_t *u16vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API int16_t  *s16vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API uint32_t *u32vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API int32_t  *s32vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API uint64_t *u64vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API int64_t  *s64vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API float    *f32vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API double   *f64vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API std::complex<float>  *c32vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
-PMT_API std::complex<double> *c64vector_writable_elements(pmt_t v, size_t &len); //< len is in elements
+PMT_API uint8_t* u8vector_writable_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API int8_t* s8vector_writable_elements(pmt_t v, size_t& len);  //< len is in elements
+PMT_API uint16_t* u16vector_writable_elements(pmt_t v,
+                                              size_t& len);         //< len is in elements
+PMT_API int16_t* s16vector_writable_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API uint32_t* u32vector_writable_elements(pmt_t v,
+                                              size_t& len);         //< len is in elements
+PMT_API int32_t* s32vector_writable_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API uint64_t* u64vector_writable_elements(pmt_t v,
+                                              size_t& len);         //< len is in elements
+PMT_API int64_t* s64vector_writable_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API float* f32vector_writable_elements(pmt_t v, size_t& len);   //< len is in elements
+PMT_API double* f64vector_writable_elements(pmt_t v, size_t& len);  //< len is in elements
+PMT_API std::complex<float>*
+c32vector_writable_elements(pmt_t v, size_t& len); //< len is in elements
+PMT_API std::complex<double>*
+c64vector_writable_elements(pmt_t v, size_t& len); //< len is in elements
 
 /*
  * ------------------------------------------------------------------------
@@ -576,22 +655,22 @@ PMT_API std::complex<double> *c64vector_writable_elements(pmt_t v, size_t &len);
  */
 
 //! Return true if \p obj is a dictionary (warning: also returns true for a pair)
-PMT_API bool is_dict(const pmt_t &obj);
+PMT_API bool is_dict(const pmt_t& obj);
 
 //! Make an empty dictionary
 PMT_API pmt_t make_dict();
 
 //! Return a new dictionary with \p key associated with \p value.
-PMT_API pmt_t dict_add(const pmt_t &dict, const pmt_t &key, const pmt_t &value);
+PMT_API pmt_t dict_add(const pmt_t& dict, const pmt_t& key, const pmt_t& value);
 
 //! Return a new dictionary with \p key removed.
-PMT_API pmt_t dict_delete(const pmt_t &dict, const pmt_t &key);
+PMT_API pmt_t dict_delete(const pmt_t& dict, const pmt_t& key);
 
 //! Return true if \p key exists in \p dict
-PMT_API bool  dict_has_key(const pmt_t &dict, const pmt_t &key);
+PMT_API bool dict_has_key(const pmt_t& dict, const pmt_t& key);
 
 //! If \p key exists in \p dict, return associated value; otherwise return \p not_found.
-PMT_API pmt_t dict_ref(const pmt_t &dict, const pmt_t &key, const pmt_t &not_found);
+PMT_API pmt_t dict_ref(const pmt_t& dict, const pmt_t& key, const pmt_t& not_found);
 
 //! Return list of (key . value) pairs
 PMT_API pmt_t dict_items(pmt_t dict);
@@ -600,7 +679,7 @@ PMT_API pmt_t dict_items(pmt_t dict);
 PMT_API pmt_t dict_keys(pmt_t dict);
 
 //! Return a new dictionary \p dict1 with k=>v pairs from \p dict2 added.
-PMT_API pmt_t dict_update(const pmt_t &dict1, const pmt_t &dict2);
+PMT_API pmt_t dict_update(const pmt_t& dict1, const pmt_t& dict2);
 
 //! Return list of values
 PMT_API pmt_t dict_values(pmt_t dict);
@@ -618,13 +697,13 @@ PMT_API pmt_t dict_values(pmt_t dict);
 PMT_API bool is_any(pmt_t obj);
 
 //! make an any
-PMT_API pmt_t make_any(const boost::any &any);
+PMT_API pmt_t make_any(const boost::any& any);
 
 //! Return underlying boost::any
 PMT_API boost::any any_ref(pmt_t obj);
 
 //! Store \p any in \p obj
-PMT_API void any_set(pmt_t obj, const boost::any &any);
+PMT_API void any_set(pmt_t obj, const boost::any& any);
 
 
 /*
@@ -633,13 +712,13 @@ PMT_API void any_set(pmt_t obj, const boost::any &any);
  * ------------------------------------------------------------------------
  */
 //! Return true if \p obj is a msg_accepter
-PMT_API bool is_msg_accepter(const pmt_t &obj);
+PMT_API bool is_msg_accepter(const pmt_t& obj);
 
 //! make a msg_accepter
 PMT_API pmt_t make_msg_accepter(boost::shared_ptr<gr::messages::msg_accepter> ma);
 
 //! Return underlying msg_accepter
-PMT_API boost::shared_ptr<gr::messages::msg_accepter> msg_accepter_ref(const pmt_t &obj);
+PMT_API boost::shared_ptr<gr::messages::msg_accepter> msg_accepter_ref(const pmt_t& obj);
 
 /*
  * ------------------------------------------------------------------------
@@ -651,7 +730,8 @@ PMT_API boost::shared_ptr<gr::messages::msg_accepter> msg_accepter_ref(const pmt
 PMT_API bool eq(const pmt_t& x, const pmt_t& y);
 
 /*!
- * \brief Return true if x and y should normally be regarded as the same object, else false.
+ * \brief Return true if x and y should normally be regarded as the same object, else
+ * false.
  *
  * <pre>
  * eqv returns true if:
@@ -733,11 +813,7 @@ PMT_API pmt_t reverse_x(pmt_t list);
 /*!
  * \brief (acons x y a) == (cons (cons x y) a)
  */
-inline static pmt_t
-acons(pmt_t x, pmt_t y, pmt_t a)
-{
-  return cons(cons(x, y), a);
-}
+inline static pmt_t acons(pmt_t x, pmt_t y, pmt_t a) { return cons(cons(x, y), a); }
 
 /*!
  * \brief locates \p nth element of \n list where the car is the 'zeroth' element.
@@ -772,8 +848,8 @@ PMT_API pmt_t memv(pmt_t obj, pmt_t list);
 PMT_API pmt_t member(pmt_t obj, pmt_t list);
 
 /*!
- * \brief Return true if every element of \p list1 appears in \p list2, and false otherwise.
- * Comparisons are done with pmt::eqv.
+ * \brief Return true if every element of \p list1 appears in \p list2, and false
+ * otherwise. Comparisons are done with pmt::eqv.
  */
 PMT_API bool subsetp(pmt_t list1, pmt_t list2);
 
@@ -800,13 +876,19 @@ PMT_API pmt_t list4(const pmt_t& x1, const pmt_t& x2, const pmt_t& x3, const pmt
 /*!
  * \brief Return a list of length 5 containing \p x1, \p x2, \p x3, \p x4, \p x5
  */
-PMT_API pmt_t list5(const pmt_t& x1, const pmt_t& x2, const pmt_t& x3, const pmt_t& x4, const pmt_t& x5);
+PMT_API pmt_t list5(
+    const pmt_t& x1, const pmt_t& x2, const pmt_t& x3, const pmt_t& x4, const pmt_t& x5);
 
 /*!
  * \brief Return a list of length 6 containing \p x1, \p x2, \p x3, \p x4, \p
  * x5, \p x6
  */
-PMT_API pmt_t list6(const pmt_t& x1, const pmt_t& x2, const pmt_t& x3, const pmt_t& x4, const pmt_t& x5, const pmt_t& x6);
+PMT_API pmt_t list6(const pmt_t& x1,
+                    const pmt_t& x2,
+                    const pmt_t& x3,
+                    const pmt_t& x4,
+                    const pmt_t& x5,
+                    const pmt_t& x6);
 
 /*!
  * \brief Return \p list with \p item added to it.
@@ -848,12 +930,12 @@ PMT_API bool is_eof_object(pmt_t obj);
  * representation, but the external representation is incomplete and
  * therefore not parsable, an error is signaled.
  */
-PMT_API pmt_t read(std::istream &port);
+PMT_API pmt_t read(std::istream& port);
 
 /*!
  * Write a written representation of \p obj to the given \p port.
  */
-PMT_API void write(pmt_t obj, std::ostream &port);
+PMT_API void write(pmt_t obj, std::ostream& port);
 
 /*!
  * Return a string representation of \p obj.
@@ -862,7 +944,7 @@ PMT_API void write(pmt_t obj, std::ostream &port);
 PMT_API std::string write_string(pmt_t obj);
 
 
-PMT_API std::ostream& operator<<(std::ostream &os, pmt_t obj);
+PMT_API std::ostream& operator<<(std::ostream& os, pmt_t obj);
 
 /*!
  * \brief Write pmt string representation to stdout.
@@ -878,15 +960,15 @@ PMT_API void print(pmt_t v);
 /*!
  * \brief Write portable byte-serial representation of \p obj to \p sink
  */
-PMT_API bool serialize(pmt_t obj, std::streambuf &sink);
+PMT_API bool serialize(pmt_t obj, std::streambuf& sink);
 
 /*!
  * \brief Create obj from portable byte-serial representation
  */
-PMT_API pmt_t deserialize(std::streambuf &source);
+PMT_API pmt_t deserialize(std::streambuf& source);
 
 
-PMT_API void dump_sizeof();	// debugging
+PMT_API void dump_sizeof(); // debugging
 
 /*!
  * \brief Provide a simple string generating interface to pmt's serialize function
@@ -901,17 +983,23 @@ PMT_API pmt_t deserialize_str(std::string str);
 /*!
  * \brief Provide a comparator function object to allow pmt use in stl types
  */
-class comparator {
-    public:
-        bool operator()(pmt::pmt_t const& p1, pmt::pmt_t const& p2) const
-            { return pmt::eqv(p1,p2)?false:p1.get()>p2.get(); }
+class comparator
+{
+public:
+    bool operator()(pmt::pmt_t const& p1, pmt::pmt_t const& p2) const
+    {
+        return pmt::eqv(p1, p2) ? false : p1.get() > p2.get();
+    }
 };
 
 // FIXME: Remove in 3.8.
-class comperator {
-    public:
-        bool operator()(pmt::pmt_t const& p1, pmt::pmt_t const& p2) const
-            { return pmt::eqv(p1,p2)?false:p1.get()>p2.get(); }
+class comperator
+{
+public:
+    bool operator()(pmt::pmt_t const& p1, pmt::pmt_t const& p2) const
+    {
+        return pmt::eqv(p1, p2) ? false : p1.get() > p2.get();
+    }
 };
 
 } /* namespace pmt */
