@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
 #
 # Copyright 2015 Free Software Foundation, Inc.
 #
@@ -19,4 +21,4 @@
 # Boston, MA 02110-1301, USA.
 #
 
-from Generate_LDPC_matrix_functions import *
+from .Generate_LDPC_matrix_functions import *
